/*
 * Copyright (c) 2008, Willow Garage, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Willow Garage, Inc. nor the names of its
 *       contributors may be used to endorse or promote products derived from
 *       this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * \file
 * 
 * Utilities for occupancy grids represented as boost multiarrays of bools 
 */

#ifndef TOPOLOGICAL_MAP_OCCUPANCY_GRID_H
#define TOPOLOGICAL_MAP_OCCUPANCY_GRID_H

#include <costmap_2d/costmap_2d.h>
#include <boost/multi_array.hpp>
#include <boost/shared_ptr.hpp>

namespace topological_map
{

/************************************************************
 * Types
 ************************************************************/

typedef boost::multi_array<bool, 2> OccupancyGrid;
typedef boost::shared_ptr<OccupancyGrid> OccGridPtr;




/************************************************************
 * Operations
 ************************************************************/

/// Number of columns in a grid
unsigned numCols(const OccupancyGrid& grid);

/// Number of rows in a grid
unsigned numRows(const OccupancyGrid& grid);

/// Convert a ros costmap into an occupancy grid.
/// A cell is an obstacle iff its a lethal or inflated obstacle, or unknown (NO_INFORMATION)
OccupancyGrid costmapToGrid (const costmap_2d::Costmap2D& cmap);

// Convert an occupancy grid into a ros costmap.
// The resolution is 1 and the origin is 0, 0.  Obstacles are made lethal.
costmap_2d::Costmap2D gridToCostmap (const OccupancyGrid& grid);









} // namespace


#endif // include_guard
