/*
 * Copyright (c) 2008, Willow Garage, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Willow Garage, Inc. nor the names of its
 *       contributors may be used to endorse or promote products derived from
 *       this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

/**
 * \file 
 * 
 * utilities
 */

#ifndef TOPOLOGICAL_MAP_UTILS_H
#define TOPOLOGICAL_MAP_UTILS_H

#include <boost/lexical_cast.hpp>
#include <string>
#include <numeric>

namespace topological_map
{

template <class K, class C>
bool contains (const C& container, const K& key)
{
  return container.find(key)!=container.end();
}

template <class K>
bool contains (const vector<K>& container, const K& key)
{
  return find(container.begin(), container.end(), key) != container.end();
}



inline bool closeTo (const double x, const double y)
{
  return abs(x-y)<1e-6;
}


template <class T>
std::string concatenate (const std::string& s, const T& x)
{
  return s + std::string(" ") + boost::lexical_cast<std::string>(x);
}

template <class T>
std::string toString (const std::set<T>& set)
{
  const std::string s = std::accumulate(set.begin(), set.end(), std::string(""), concatenate<T>);
  return std::string("[") + s + std::string("]");
}

template <class T>
std::string toString (const std::vector<T>& set)
{
  const std::string s = std::accumulate(set.begin(), set.end(), std::string(""), concatenate<T>);
  return std::string("[") + s + std::string("]");
}



} // namespace topological_map



#endif // include guard
