/*
 * Copyright (c) 2008, Willow Garage, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Willow Garage, Inc. nor the names of its
 *       contributors may be used to endorse or promote products derived from
 *       this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * \file
 * 
 * Implementation of occupancy_grid.h
 */


#include <topological_map/occupancy_grid.h>


namespace topological_map
{

namespace cmap=costmap_2d;
using cmap::Costmap2D;

OccupancyGrid costmapToGrid (const Costmap2D& cmap)
{
  
  const unsigned nx = cmap.getSizeInCellsX();
  const unsigned ny = cmap.getSizeInCellsY();
  
  OccupancyGrid grid(boost::extents[nx][ny]);
  for (unsigned x=0; x<nx; ++x) {
    for (unsigned y=0; y<ny; ++y) {
      unsigned char cost = cmap.getCost(x,y);

      // Leave out those points that are lethal or inscribed_inflated obstacles
      grid[x][y] = (cost == cmap::INSCRIBED_INFLATED_OBSTACLE) || (cost == cmap::LETHAL_OBSTACLE)
        || (cost == cmap::NO_INFORMATION);        
    }
  }
  return grid;
}

Costmap2D gridToCostmap (const OccupancyGrid& grid)
{
  const unsigned nr = numRows(grid);
  const unsigned nc = numCols(grid);

  Costmap2D cmap(nr, nc, 1.0, 0.0, 0.0);
  for (unsigned r=0; r<nr; r++) 
    for (unsigned c=0; c<nc; c++)
      cmap.setCost(r, c, grid[r][c] ? cmap::LETHAL_OBSTACLE : cmap::FREE_SPACE);

  return cmap;
}

unsigned numRows(const OccupancyGrid& grid)
{
  const OccupancyGrid::size_type* dims=grid.shape();
  return dims[0];
}

unsigned numCols(const OccupancyGrid& grid)
{
  const OccupancyGrid::size_type* dims=grid.shape();
  return dims[1];
}






} // namespace
