/*
 * Copyright (c) 2008, Willow Garage, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Willow Garage, Inc. nor the names of its
 *       contributors may be used to endorse or promote products derived from
 *       this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/// \file
/// 
/// Implements ops in point_ops.h

#include <topological_map/point_ops.h>

namespace topological_map
{

int operator< (const Point& p, const Point& p2)
{
  return (p.x<p2.x) || ((p.x==p2.x) && (p.y<p2.y)) ||
    ((p.x==p2.x) && (p.y==p2.y) && (p.z<p2.z));
}

bool operator== (const Point& p, const Point& p2)
{
  return (p.x==p2.x) && (p.y==p2.y) && (p.z==p2.z);
}

ostream& operator<< (ostream& str, const Point& p)
{
  str << "(" << p.x << ", " << p.y << ")";
  return str;
}

Point makePoint (const double x, const double y, const double z)
{
  Point p;
  p.x = x;
  p.y = y;
  p.z = z;
  return p;
}

double euclideanDistance (const Point& p1, const Point& p2)
{
  const double dx = p1.x-p2.x;
  const double dy = p1.y-p2.y;
  const double dz = p1.z-p2.z;
  return sqrt(dx*dx + dy*dy + dz*dz);
}






} // namespace
